// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 hTex: TEXTUREMATRIX <string uiname="Highlight Transform";>;
float2 chooser;
float2 selected;
bool mouseover;
float cols;
float rows;
float4 cAmb : COLOR <String uiname="Color";>  = {0, 0, 0, 1};

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 TexOrg : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    float4 TexOrg = 0;
    Out.TexOrg = mul(TexCd, hTex);

    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    chooser.y = chooser.y*(-1);
    chooser = chooser/2+0.5;
    float ch_xstart = floor(chooser.x*cols)/cols;
    float ch_xend = ch_xstart+1/cols;

    float ch_ystart = floor(chooser.y*rows)/rows;
    float ch_yend = ch_ystart+1/rows;
    float4 col = tex2D(Samp, In.TexCd);
    if ( In.TexOrg.x > ch_xstart && In.TexOrg.x < ch_xend && In.TexOrg.y > ch_ystart && In.TexOrg.y < ch_yend ) {
    col.r += 0.35;
    col.g += 0.15;
    } else {
    col = tex2D(Samp, In.TexCd)*cAmb;
    }
    
    selected.y = selected.y*(-1);
    selected = selected/2+0.5;
    float s_xstart = floor(selected.x*cols)/cols;
    float s_xend = s_xstart+1/cols;

    float s_ystart = floor(selected.y*rows)/rows;
    float s_yend = s_ystart+1/rows;
    if ( In.TexOrg.x > s_xstart && In.TexOrg.x < s_xend && In.TexOrg.y > s_ystart && In.TexOrg.y < s_yend ) {
    col.r += 0.5;
    }
    if (mouseover == 0)
    { col.a -=0.25;
    }
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Highlighter
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
